<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\WebHooks\Handlers;

use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Authorization\Exceptions\FailedToRefreshAccessToken;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Authorization\Exceptions\FailedToRetrieveAuthInfoException;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Receiver\Tasks\Composite\UnsubscribeReceiverTask;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Http\Exceptions\HttpCommunicationException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Http\Exceptions\HttpRequestException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\AbortTaskExecutionException;
use Exception;

/**
 * Class Receiver_Unsubscribed_Handler
 *
 * @package CleverReach\WooCommerce\Components\WebHooks\Handlers
 */
class Receiver_Unsubscribed_Handler extends Handler {

	/**
	 * Runs task logic.
	 *
	 * @inheritDoc
	 * @throws Exception Exception.
	 */
	public function execute() {
		$receiver   = $this->get_receiver( $this->get_group_service()->getId(), $this->receiver_id );
		$subscriber = $this->get_subscriber_service()->getReceiver( $receiver->getEmail() );

		if ( null === $subscriber ) {
			throw new AbortTaskExecutionException(
				sprintf(
					'Receiver [%s] cannot be unsubscribed.',
					$receiver->getEmail()
				)
			);
		}

		$this->reportProgress( 60 );
		$this->handle_subscriber_update_or_create_event( $receiver, $subscriber );
		$this->enqueue( new UnsubscribeReceiverTask( $receiver->getEmail() ) );
		$this->reportProgress( 100 );
	}
}
