<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\WebHooks\Listeners;

use CleverReach\WooCommerce\Components\Services\BusinessLogic\Configuration\Config_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Configuration\Configuration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Configuration\ConfigurationManager;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueService;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Task;

/**
 * Class Listener
 *
 * @package CleverReach\WooCommerce\Components\WebHooks\Listeners
 */
abstract class Listener {

	/**
	 * Enqueues task.
	 *
	 * @param Task $task Task.
	 *
	 * @throws QueueStorageUnavailableException Exception if queue storage unavailable.
	 */
	protected static function enqueue( Task $task ) {
		$context   = ConfigurationManager::getInstance()->getContext();
		$queue_name = static::get_config_service()->getDefaultQueueName();

		static::get_queue()->enqueue( $queue_name, $task, $context );
	}

	/**
	 * Retrieves config service.
	 *
	 * @return Config_Service
	 */
	protected static function get_config_service() {
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return ServiceRegister::getService( Configuration::CLASS_NAME );
	}

	/**
	 * Retrieves queue service.
	 *
	 * @return QueueService
	 */
	protected static function get_queue() {
		/** @noinspection PhpIncompatibleReturnTypeInspection */
		return ServiceRegister::getService( QueueService::CLASS_NAME );
	}
}
