<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WooCommerce\Components\Services\BusinessLogic\Support_Service;
use CleverReach\WooCommerce\Components\Util\HTTP_Helper;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\SupportConsole\Contracts\SupportService;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Exceptions\QueryFilterInvalidParamException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Exceptions\RepositoryNotRegisteredException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Clever_Reach_Support_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Support_Controller extends Clever_Reach_Base_Controller {
	/**
	 * Support service.
	 *
	 * @var Support_Service
	 */
	private $support_service;

	/**
	 * Return configuration.
	 */
	public function display() {
		$this->return_json( array( $this->get_support_service()->get() ) );
	}

	/**
	 * Updates configuration
	 *
	 * @throws QueryFilterInvalidParamException Wxception if query filter params are invalid.
	 * @throws RepositoryNotRegisteredException Exception if repository not registered.
	 */
	public function modify() {
		$body = HTTP_Helper::get_body();

		$this->return_json( array( $this->get_support_service()->update( $body ) ) );
	}

	/**
	 * Returns support service.
	 *
	 * @return Support_Service
	 */
	private function get_support_service() {
		if ( null === $this->support_service ) {
			$this->support_service = ServiceRegister::getService( SupportService::CLASS_NAME );
		}

		return $this->support_service;
	}
}
