<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\ViewModel\InitialSyncSettings;

use CleverReach\WooCommerce\Components\Repositories\Customers\Buyer_Repository;
use CleverReach\WooCommerce\Components\Repositories\Customers\Contact_Repository;
use CleverReach\WooCommerce\Components\Repositories\Customers\Subscriber_Repository;
use CleverReach\WooCommerce\Components\Services\BusinessLogic\Language\Translation_Service;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Language\Contracts\TranslationService as Base_Translation_Service;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

/**
 * Class Receivers_Data
 *
 * @package CleverReach\WooCommerce\ViewModel\InitialSyncSettings
 */
class Receivers_Data {

	/**
	 * Subscriber repository
	 *
	 * @var Subscriber_Repository
	 */
	private $subscriber_repository;

	/**
	 * Buyer repository
	 *
	 * @var Buyer_Repository
	 */
	private $buyer_repository;

	/**
	 * Contact repository
	 *
	 * @var Contact_Repository
	 */
	private $contact_repository;

	/**
	 * Translation service
	 *
	 * @var Translation_Service
	 */
	private $translation_service;

	/**
	 * ReceiversData constructor.
	 */
	public function __construct() {
		$this->subscriber_repository = new Subscriber_Repository();
		$this->buyer_repository      = new Buyer_Repository();
		$this->contact_repository    = new Contact_Repository();
	}

	/**
	 * Checks if subscribers exist in system
	 *
	 * @return string
	 */
	public function get_subscribers_exist() {
		return $this->subscriber_repository->count() > 0 ? '1' : '0';
	}

	/**
	 * Returns number of subscribers
	 *
	 * @return string
	 */
	public function get_subscribers_count() {
		return $this->get_translation_service()->format_number( $this->subscriber_repository->count() );
	}

	/**
	 * Returns number of buyers
	 *
	 * @return string
	 */
	public function get_buyers_count() {
		return $this->get_translation_service()->format_number( $this->buyer_repository->count() );
	}

	/**
	 * Returns number of contacts
	 *
	 * @return string
	 */
	public function get_contacts_count() {
		return $this->get_translation_service()->format_number( $this->contact_repository->count() );
	}

	/**
	 * Returns translation service
	 *
	 * @return Translation_Service
	 */
	private function get_translation_service() {
		if ( null === $this->translation_service ) {
			$this->translation_service = ServiceRegister::getService( Base_Translation_Service::CLASS_NAME );
		}

		return $this->translation_service;
	}
}
