(function () {
    jQuery(document).ready(function () {
        const loader = document.getElementById('crLoader');

        jQuery('#tabs-container').tabs({
            create: function (event, ui) {
                ui.tab[0].firstElementChild.classList.add('nav-tab-active');
            },
            activate: function (event, ui) {
                ui.newTab[0].firstElementChild.classList.add('nav-tab-active');
                ui.oldTab[0].firstElementChild.classList.remove('nav-tab-active');
                switch (ui.newTab[0].id) {
                    case 'cr-tab-settings':
                        if (!secondarySyncOngoing) {
                            fetchCurrentSettings();
                            setButtonsState();
                            setNewsletterSettingsState();
                        }
                        break;
                    default:
                }
            }
        });

        jQuery("#dialog").dialog({
            dialogClass: "no-close",
            resizable: false,
            height: "auto",
            width: 400,
            modal: true,
            buttons: [
                {
                    text: document.getElementById('cr-confirm-translation').value,
                    class: "button cr-primary",
                    click: function () {
                        const uninstallUrl = document.getElementById('crUninstallUrl').value;
                        sendAjax(uninstallUrl, null, true);
                        jQuery(this).dialog("close");
                    }
                },
                {
                    text: document.getElementById('cr-cancel-translation').value,
                    class: "button cr-secondary",
                    click: function () {
                        jQuery(this).dialog("close");
                    }
                }
            ]
        }).dialog('close');

        // DASHBOARD
        const helpUrl = document.getElementById('crHelpUrl').value;

        const retrySyncUrl = document.getElementById('crRetrySyncUrl').value;
        const redirectUrl = document.getElementById('crRedirectUrl').value;

        const blogButton = document.getElementById('crPush');
        const helpButton = document.getElementById('crHelp');
        const disconnectButton = document.getElementById('crDisconnect');

        const retrySynchronization = document.getElementById('crRetrySync');

        if (retrySynchronization) {
            retrySynchronization.addEventListener('click', function () {
                sendAjax(retrySyncUrl, null, true);
            });
        }

        const buttons = [
            document.getElementById('crBuildEmail'),
            document.getElementById('crReports'),
            document.getElementById('crForms'),
            document.getElementById('crThea'),
            document.getElementById('crEmails'),
            document.getElementById('crPricePlan'),
        ];

        for (const b of buttons) {
            if (b) {
                b.addEventListener('click', function () {
                    openOnCleverReach(redirectUrl, {param: b.dataset.url});
                });
            }
        }

        if (blogButton) {
            blogButton.addEventListener('click', function () {
                window.open(blogButton.dataset.url);
            })
        }

        helpButton.addEventListener('click', function () {
            const win = window.open(helpUrl, '_blank');
            win.focus();
        });

        disconnectButton.addEventListener('click', function () {
            jQuery("#dialog").dialog('open');
        });

        // Settings
        const saveSyncSettingsUrl = document.querySelector('#crSaveSyncSettingsUrl');
        const saveNewsletterSettingsUrl = document.querySelector('#crSaveNewsletterSettingsUrl');
        const fetchSettingsUrl = document.querySelector('#crFetchSettingsUrl');
        const checkStatusUrl = document.querySelector('#crCheckSecondarySyncStatusUrl');
        const reforceSyncUrl = document.querySelector('#crReforceSyncUrl');

        const syncErrorPanel = document.querySelector('#syncErrorPanel');
        const backgroundSyncPanel = document.querySelector('#backgroundSyncPanel');

        const subscribers = document.querySelector('#crSubscribersCheckbox');
        let buyersCheckbox = document.querySelector('#crBuyersCheckbox');
        let contactsCheckbox = document.querySelector('#crContactsCheckbox');
        let newsletterEnabledCheckbox = document.querySelector('#crNewsletterEnabled');
        let newsletterCheckboxCaption = document.querySelector('#crNewsletterCaption');
        let doiEnabledCheckbox = document.querySelector('#crDoiEnabled');
        let defaultFormSelect = document.querySelector('#crDefaultForm');
        const isUserDataComplete = document.querySelector('#crIsUserDataComplete').value === '1';

        const saveButton = document.querySelector('#crSettingsSaveButton');
        const reforceSyncButton = document.querySelector('#crInitialSyncSettingsReforceSyncButton');

        const initialSettings = {
            syncSettings: {
                subscribers: true,
                buyers: false,
                contacts: false
            },
            newsletterSettings: {
                isNewsletterCheckboxEnabled: newsletterEnabledCheckbox.checked,
                newsletterCheckboxCaption: newsletterCheckboxCaption.value,
                isDoiEnabled: doiEnabledCheckbox.checked,
                defaultForm: defaultFormSelect.value
            }
        };

        let secondarySyncOngoing = false;
        let settingsHasChanged = false;

        function fetchCurrentSettings() {
            loader.classList.remove('cr-hidden');

            CleverReach.Ajax.get(fetchSettingsUrl.value, null, function (response) {
                loader.classList.add('cr-hidden');
                if (response) {
                    buyersCheckbox.checked = response['buyers'];
                    contactsCheckbox.checked = response['contacts'];

                    initialSettings.syncSettings.buyers = response['buyers'];
                    initialSettings.syncSettings.contacts = response['contacts'];

                    setButtonsState();
                }
            }, 'json', true);
        }

        /**
         * Disables import button if settings have changed
         */
        function setButtonsState() {
            isSettingsChanged();
            saveButton.disabled = !settingsHasChanged;
        }

        function startSecSyncState() {
            secondarySyncOngoing = true;
            reforceSyncButton.classList.add('is-loading');
            reforceSyncButton.disabled = true;
            reforceSyncButton.style.pointerEvents = 'none';
        }

        function endSecSyncState() {
            secondarySyncOngoing = false;
            reforceSyncButton.classList.remove('is-loading');
            reforceSyncButton.disabled = false;
            reforceSyncButton.style.pointerEvents = '';
        }

        function secSyncHandler() {
            startSecSyncState();

            CleverReach.Ajax.post(reforceSyncUrl.value, null, function (response) {
                if (response.success) {
                    setSyncSettingsFormToReadOnlyState();
                    checkSecondarySync();
                } else {
                    endSecSyncState();
                    setFormToFailureState();
                }
            }, 'json', true);
        }

        reforceSyncButton.addEventListener('click', secSyncHandler);

        function checkSecondarySync() {
            setSyncSettingsFormToReadOnlyState();
            if (secondarySyncOngoing) {
                showBackgroundSyncPanel();
            }

            CleverReach.Ajax.get(checkStatusUrl.value, null, function (response) {
                if (response.status === 'completed') {
                    endSecSyncState();
                    enableForm();
                } else if (response.status === 'failed') {
                    endSecSyncState();
                    setFormToFailureState();
                } else {
                    setTimeout(function () {
                        startSecSyncState();
                        checkSecondarySync();
                    }, 250);
                }
            }, 'json', true);
        }

        function showBackgroundSyncPanel() {
            if (backgroundSyncPanel.classList.contains('cr-hidden')) {
                backgroundSyncPanel.classList.remove('cr-hidden');
            }
        }

        function setFormToFailureState() {
            buyersCheckbox.style.pointerEvents = 'none';
            contactsCheckbox.style.pointerEvents = 'none';
            saveButton.disabled = true;
            syncErrorPanel.classList.remove('cr-hidden');
            backgroundSyncPanel.classList.add('cr-hidden');
        }

        function enableForm() {
            buyersCheckbox.style.pointerEvents = '';
            contactsCheckbox.style.pointerEvents = '';
            saveButton.style.pointerEvents = '';
            buyersCheckbox.disabled = false;
            contactsCheckbox.disabled = false;
            saveButton.disabled = false;
            buyersCheckbox.classList.remove('cr-disabled');
            subscribers.classList.remove('cr-disabled');
            contactsCheckbox.classList.remove('cr-disabled');

            if (!syncErrorPanel.classList.contains('cr-hidden')) {
                syncErrorPanel.classList.add('cr-hidden');
            }
            if (!backgroundSyncPanel.classList.contains('cr-hidden')) {
                backgroundSyncPanel.classList.add('cr-hidden');
            }

            setButtonsState();
        }

        function setSyncSettingsFormToReadOnlyState() {
            buyersCheckbox.style.pointerEvents = 'none';
            contactsCheckbox.style.pointerEvents = 'none';
            saveButton.style.pointerEvents = 'none';
            buyersCheckbox.disabled = true;
            contactsCheckbox.disabled = true;
            saveButton.disabled = true;
            buyersCheckbox.classList.add('cr-disabled');
            subscribers.classList.add('cr-disabled');
            contactsCheckbox.classList.add('cr-disabled');
        }

        if (buyersCheckbox) {
            buyersCheckbox.addEventListener('change', function () {
                if (buyersCheckbox.checked === false) {
                    contactsCheckbox.checked = false;
                }
                setButtonsState();
            });
        }

        if (contactsCheckbox) {
            contactsCheckbox.addEventListener('change', function () {
                if (contactsCheckbox.checked) {
                    buyersCheckbox.checked = true;
                }
                setButtonsState();
            });
        }

        function startSaveSettingsState() {
            loader.classList.remove('cr-hidden');
        }

        function endSaveSettingsState() {
            loader.classList.add('cr-hidden');
        }

        function saveSettingsHandler() {
            startSaveSettingsState();
            setSyncSettingsFormToReadOnlyState();

            const selectedOptions = ['subscribers'];

            if (buyersCheckbox.checked) {
                selectedOptions.push('buyers');
            }

            if (contactsCheckbox.checked) {
                selectedOptions.push('contacts');
            }
            const body = {
                'syncSettings': selectedOptions,
            };

            CleverReach.Ajax.post(saveSyncSettingsUrl.value, body, (response) => {
                endSaveSettingsState();
                if (response.success) {
                    /**
                     * Check if reforce sync should also be run
                     **/
                    const selectedOptionsLength = selectedOptions.length;
                    let initialSettingsLength = 0;
                    const initialSettingsValues = Object.values(initialSettings.syncSettings);
                    for (const optionValue of initialSettingsValues) {
                        if (optionValue) {
                            initialSettingsLength++;
                        }
                    }
                    initialSettings.syncSettings.buyers = buyersCheckbox.checked;
                    initialSettings.syncSettings.contacts = contactsCheckbox.checked;
                    if (selectedOptionsLength > initialSettingsLength) {
                        reforceSyncButton.click();
                    }

                    checkSecondarySync();
                } else {
                    enableForm();
                }
            }, 'json', true);

            const newsletterSettingsBody = {
                'isNewsletterCheckboxEnabled': newsletterEnabledCheckbox.checked,
                'newsletterCheckboxCaption': newsletterEnabledCheckbox.checked ? newsletterCheckboxCaption.value : false,
                'isDoiEnabled': !!doiEnabledCheckbox.checked,
                'defaultForm': defaultFormSelect.value || false,
            };

            // Save Newsletter settings
            CleverReach.Ajax.post(saveNewsletterSettingsUrl.value, newsletterSettingsBody, (response) => {
                if (response.success) {
                    initialSettings.newsletterSettings.isNewsletterCheckboxEnabled = newsletterEnabledCheckbox.checked;
                    initialSettings.newsletterSettings.newsletterCheckboxCaption = newsletterCheckboxCaption.value;
                    initialSettings.newsletterSettings.isDoiEnabled = doiEnabledCheckbox.checked;
                    initialSettings.newsletterSettings.defaultForm = defaultFormSelect.value;
                }
                setButtonsState();
            }, 'json', true);
        }

        if (saveButton) {
            saveButton.addEventListener('click', saveSettingsHandler);
        }

        function setNewsletterSettingsState() {
            newsletterCheckboxCaption.disabled = true;
            doiEnabledCheckbox.disabled = true;
            defaultFormSelect.disabled = true;
            if (newsletterEnabledCheckbox.checked) {
                newsletterCheckboxCaption.disabled = false;
                if (isUserDataComplete) {
                    doiEnabledCheckbox.disabled = false;
                    if (doiEnabledCheckbox.checked) {
                        defaultFormSelect.disabled = false;
                    }
                }
            }
            setButtonsState();
        }

        newsletterEnabledCheckbox.addEventListener('change', setNewsletterSettingsState);
        newsletterCheckboxCaption.addEventListener('input', setButtonsState);
        doiEnabledCheckbox.addEventListener('change', setNewsletterSettingsState);
        defaultFormSelect.addEventListener('change', setButtonsState);

        function isSettingsChanged() {
            if (
                initialSettings.syncSettings.buyers !== buyersCheckbox.checked ||
                initialSettings.syncSettings.contacts !== contactsCheckbox.checked ||
                initialSettings.newsletterSettings.isNewsletterCheckboxEnabled !== newsletterEnabledCheckbox.checked ||
                initialSettings.newsletterSettings.newsletterCheckboxCaption !== newsletterCheckboxCaption.value.trim() ||
                initialSettings.newsletterSettings.isDoiEnabled !== doiEnabledCheckbox.checked ||
                initialSettings.newsletterSettings.defaultForm !== defaultFormSelect.value
            ) {
                settingsHasChanged = true;
                return
            }
            settingsHasChanged = false;
        }

        // General
        function openOnCleverReach(url, data) {
            CleverReach.Ajax.post(url, data, function (response) {
                if (response.url) {
                    const win = window.open(response.url, '_blank');
                    win.focus();
                }
            }, 'json', true);
        }

        function sendAjax(url, body = null, hasLoader = false) {
            if (hasLoader) {
                loader.classList.remove('cr-hidden');
            }
            CleverReach.Ajax.post(url, body, function (response) {
                if (response.success === true) {
                    location.reload();
                }
            }, 'json', true);
        }
    });
})();
