<?php

namespace CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Automation\AbandonedCart\Events;

use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Automation\AbandonedCart\Contracts\AbandonedCartRecordService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Automation\AbandonedCart\Pipeline\AbandonedCartCreatePipeline;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;

class AbandonedCartEventListener
{
    public function handle(AbandonedCartEvent $event)
    {
        try {
            AbandonedCartCreatePipeline::execute($event->getTrigger());
            $this->getService()->create($event->getTrigger());
        } catch (\Exception $e) {
            Logger::logWarning($e->getMessage(), 'Core', array('trace' => $e->getTraceAsString()));
        }
    }

    /**
     * Retrieves abandoned cart record service.
     *
     * @return AbandonedCartRecordService | object
     */
    private function getService()
    {
        return ServiceRegister::getService(AbandonedCartRecordService::CLASS_NAME);
    }
}