<?php

namespace CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Entities;

use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Configuration\EntityConfiguration;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Configuration\IndexMap;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Entity;

/**
 * Class AutomationRecord
 *
 * @package CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Multistore\AbandonedCart\Entities
 */
class AutomationRecord extends Entity
{
    const CLASS_NAME = __CLASS__;
    /**
     * Id of the associated cart automation <FK>.
     *
     * @var int
     */
    protected $automationId;
    /**
     * Group that will be used to trigger an automation.
     *
     * @var string
     */
    protected $groupId;
    /**
     * Email of the receiver (poolId can be substituted).
     *
     * @var string
     */
    protected $email;
    /**
     * System cart id.
     *
     * @var string
     */
    protected $cartId;
    /**
     * Id of the associated schedule <FK>
     *
     * @var int
     */
    protected $scheduleId;
    /**
     * Array of field names.
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'automationId',
        'groupId',
        'email',
        'cartId',
        'scheduleId',
    );

    /**
     * @return int
     */
    public function getAutomationId()
    {
        return $this->automationId;
    }

    /**
     * @param int $automationId
     */
    public function setAutomationId($automationId)
    {
        $this->automationId = $automationId;
    }

    /**
     * @return string
     */
    public function getGroupId()
    {
        return $this->groupId;
    }

    /**
     * @param string $groupId
     */
    public function setGroupId($groupId)
    {
        $this->groupId = $groupId;
    }

    /**
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * @return string
     */
    public function getCartId()
    {
        return $this->cartId;
    }

    /**
     * @param string $cartId
     */
    public function setCartId($cartId)
    {
        $this->cartId = $cartId;
    }

    /**
     * @return int
     */
    public function getScheduleId()
    {
        return $this->scheduleId;
    }

    /**
     * @param int $scheduleId
     */
    public function setScheduleId($scheduleId)
    {
        $this->scheduleId = $scheduleId;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * Returns entity configuration object.
     *
     * @return EntityConfiguration Configuration object.
     */
    public function getConfig()
    {
        $indexMap = new IndexMap();
        $indexMap->addIntegerIndex('automationId');
        $indexMap->addStringIndex('email');
        $indexMap->addStringIndex('cartId');
        $indexMap->addStringIndex('groupId');
        $indexMap->addIntegerIndex('scheduleId');

        return new EntityConfiguration($indexMap, 'AutomationRecord');
    }
}